'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global localizePage, BGcall, portInit, browser, Fennec, parseUri, translate */

// the tab object, which contains |id| and |url| (stored as unicodeUrl) of the current tab
let tab = null;

$(() => {
  localizePage();

  BGcall('recordGeneralMessage', 'popup opened');

  // Set menu entries appropriately for the selected tab.
  $('.menu-entry, .menu-status, .separator').hide();

  portInit.then(() => {
    browser.runtime.onMessage.addListener((request, sender, sendResponse) => {
      if (!(request.message === 'getCurrentTabInfoResponse')) {
        return;
      }
      // Cache tab object for later use
      const { info } = request;
      ({ tab } = info);

      const shown = {};
      function show(L) {
        L.forEach((x) => {
          shown[x] = true;
        });
      }

      function hide(L) {
        L.forEach((x) => {
          shown[x] = false;
        });
      }

      show(['div_options', 'separator2']);
      if (info.paused) {
        show(['div_status_paused', 'separator0', 'div_paused_adblock', 'div_options']);
      } else if (info.disabled_site) {
        show(['div_status_disabled', 'separator0', 'div_pause_adblock',
          'div_options']);
      } else if (info.whitelisted) {
        show(['div_status_whitelisted', 'div_enable_adblock_on_this_page',
          'separator0', 'div_pause_adblock', 'separator1',
          'div_options']);
      } else {
        show(['div_pause_adblock', 'div_blacklist', 'div_whitelist',
          'div_whitelist_page', 'div_show_resourcelist',
          'div_report_an_ad', 'separator1', 'div_options', 'separator3', 'block_counts']);

        const pageCount = info.tab_blocked || '0';
        $('#page_blocked_count').text(pageCount);
        $('#total_blocked_count').text(info.total_blocked);

        // Show help link until it is clicked.
        $('#block_counts_help')
          .toggle(info.settings.show_block_counts_help_link)
          .click(function blockCountHelpClick() {
            BGcall('setSetting', 'show_block_counts_help_link', false);
            BGcall('openTab', $(this).attr('href'));
            $(this).hide();
            if (!Fennec) {
              window.close();
            }
          });
      }

      if (Fennec) {
        hide(['div_report_an_ad', 'div_blacklist', 'div_whitelist',
          'div_show_resourcelist', 'div_report_an_ad', 'separator3']);
        // we need to set any mobile specific style elements here because the width of the desktop
        // popup menu is only 275
        const theWrapper = document.getElementById('wrapper');
        theWrapper.setAttribute('style', 'width: 100%; font-size: 20px; max-height: 100%;');
      }

      const { host } = parseUri(tab.unicodeUrl);
      const advancedOption = info.settings.show_advanced_options;
      const eligibleForUndo = !info.paused && (info.disabled_site || !info.whitelisted);

      if (info.customFilterCount > 0) {
        show(['div_undo', 'separator0']);
      }

      if (!advancedOption || !tab.id) {
        hide(['div_show_resourcelist']);
      }

      if (host === 'www.youtube.com' &&
              info.youTubeChannelName &&
              eligibleForUndo &&
              info.settings.youtube_channel_whitelist) {
        $('#div_whitelist_channel').text(translate('whitelist_youtube_channel', info.youTubeChannelName));
        show(['div_whitelist_channel']);
      }

      if (info.settings.youtube_channel_whitelist &&
              tab.unicodeUrl === 'https://www.youtube.com/feed/subscriptions') {
        show(['div_whitelist_all_channels']);
      }

      for (const div in shown) {
        if (shown[div]) {
          $(`#${div}`).show();
        }
      }

      if (!info.display_menu_stats ||
              info.paused ||
              info.disabled_site ||
              info.whitelisted) {
        $('#block_counts').hide();
      }
      sendResponse({});
    }); // end of browser.runtime.onMessage

    let tabId;
    if (window.location &&
        window.location.search) {
      const searchQuery = parseUri.parseSearch(window.location.search);
      if (searchQuery &&
          searchQuery.tabId) {
        ({ tabId } = searchQuery);
      }
    }
    if (tabId) {
      browser.runtime.sendMessage({ message: 'getCurrentTabInfo', tabId });
    } else {
      browser.runtime.sendMessage({ message: 'getCurrentTabInfo' });
    }

    // Click handlers
    $('#bugreport').click(() => {
      BGcall('recordGeneralMessage', 'bugreport clicked');
      BGcall('openTab', 'https://help.getadblock.com/support/tickets/new');
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#titletext').click(() => {
      BGcall('recordGeneralMessage', 'titletext clicked');
      BGcall('openTab', 'https://getadblock.com/');
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#div_enable_adblock_on_this_page').click(() => {
      BGcall('recordGeneralMessage', 'enable adblock clicked');
      browser.runtime.onMessage.addListener((request, sender, sendResponse) => {
        if (!(request.message === 'try_to_unwhitelist_response')) {
          return;
        }
        if (request.returnCode) {
          browser.tabs.executeScript(tab.id, { code: 'location.reload();' });
          window.close();
        } else {
          $('#div_status_whitelisted')
            .replaceWith(translate('disabled_by_filter_lists'));
        }
        sendResponse({});
      });
      BGcall('tryToUnwhitelist', tab.unicodeUrl);
    });

    $('#div_paused_adblock').click(() => {
      BGcall('recordGeneralMessage', 'unpause clicked');
      BGcall('adblockIsPaused', false);
      BGcall('handlerBehaviorChanged');
      BGcall('updateButtonUIAndContextMenus');
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#div_undo').click(() => {
      BGcall('recordGeneralMessage', 'undo clicked');
      const { host } = parseUri(tab.unicodeUrl);
      BGcall('confirmRemovalOfCustomFiltersOnHost', host, tab);
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#div_whitelist_channel').click(() => {
      BGcall('recordGeneralMessage', 'whitelist youtube clicked');
      BGcall('createWhitelistFilterForYoutubeChannel', tab.unicodeUrl);
      browser.tabs.executeScript(tab.id, { code: 'location.reload();' });
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#div_whitelist_all_channels').click(() => {
      BGcall('recordGeneralMessage', 'whitelist all youtube clicked');
      browser.tabs.sendMessage(tab.id, { type: 'whitelistAllYouTubeChannels' });
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#div_pause_adblock').click(() => {
      BGcall('recordGeneralMessage', 'pause clicked');
      BGcall('adblockIsPaused', true);
      BGcall('updateButtonUIAndContextMenus');
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#div_blacklist').click(() => {
      BGcall('recordGeneralMessage', 'blacklist clicked');
      BGcall(
        'emitPageBroadcast',
        { fn: 'topOpenBlacklistUI', options: { nothing_clicked: true } },
        { tab }, // fake sender to determine target page
      );
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#div_whitelist').click(() => {
      BGcall('recordGeneralMessage', 'whitelist domain clicked');
      BGcall(
        'emitPageBroadcast',
        { fn: 'topOpenWhitelistUI', options: {} },
        { tab }, // fake sender to determine target page
      );
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#div_whitelist_page').click(() => {
      BGcall('recordGeneralMessage', 'whitelist page clicked');
      BGcall('createPageWhitelistFilter', tab.unicodeUrl);
      browser.tabs.executeScript(tab.id, { code: 'location.reload();' });
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#div_show_resourcelist').click(() => {
      BGcall('recordGeneralMessage', 'resource clicked');
      BGcall('launchResourceblocker', `?tabId=${tab.id}`);
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#div_report_an_ad').click(() => {
      BGcall('recordGeneralMessage', 'report ad clicked');
      const url = `pages/adreport.html?url=${encodeURIComponent(tab.unicodeUrl)
      }&tabId=${tab.id}`;
      BGcall('openTab', url, true);
      if (!Fennec) {
        window.close();
      } else {
        browser.runtime.sendMessage({ message: 'closePopupTab' });
      }
    });

    $('#div_options').click(() => {
      BGcall('recordGeneralMessage', 'options clicked');
      if (!Fennec) {
        BGcall('openTab', 'options/index.html');
        window.close();
      } else {
        window.location.href = browser.runtime.getURL('options/index.html');
      }
    });

    $('#div_help_hide').click(() => {
      BGcall('recordGeneralMessage', 'help clicked');
      $('#help_hide_explanation').slideToggle();
    });

    $('#help_link').click(() => {
      BGcall('recordGeneralMessage', 'feedback clicked');
      if (!Fennec) {
        BGcall('openTab', 'http://help.getadblock.com/');
      } else {
        window.location.href = 'http://help.getadblock.com/';
      }
    });

    $('#link_open').click(() => {
      BGcall('recordGeneralMessage', 'link clicked');
      browser.storage.local.get('userid', (response) => {
        if (!Fennec) {
          BGcall('openTab', `https://getadblock.com/pay/?exp=7003&u=${response.userid}`);
          window.close();
        } else {
          window.location.href = `https://getadblock.com/mobile/pay/?exp=7003&u=${response.userid}`;
        }
      });
    });
  }); // end of portInit.then
});

